<?php
/**
 * PRO Post Types Admin
 *
 * @package  Treweler/admin
 * @version  1.14
 */

namespace TrewelerPro;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( class_exists( 'TrewelerPro\TWER_Admin_Post_Types', false ) ) {
	new TWER_Admin_Post_Types();

	return;
}

/**
 * TWER_Admin_Post_Types Class.
 * Handles the edit posts views and some functionality on the edit post screen for TWER post types.
 */
class TWER_Admin_Post_Types {

	/**
	 * Constructor.
	 */
	public function __construct() {
		include_once dirname( __FILE__ ) . '/class-twer-admin-meta-boxes.php';

		// Load correct list table classes for current screen.
		add_action( 'current_screen', [ $this, 'setup_screen' ] );
		add_action( 'check_ajax_referer', [ $this, 'setup_screen' ] );

		// Extra post data and screen elements.
		add_action( 'edit_form_after_title', [ $this, 'edit_form_after_title' ] );
		add_filter( 'enter_title_here', [ $this, 'enter_title_here' ], 1, 2 );

		// Admin notices.
		add_filter( 'post_updated_messages', [ $this, 'post_updated_messages' ] );
		add_filter( 'bulk_post_updated_messages', [ $this, 'bulk_post_updated_messages' ], 10, 2 );
	}


	/**
	 * Looks at the current screen and loads the correct list table handler.
	 *
	 * @since 3.3.0
	 */
	public function setup_screen() {
		global $twer_list_table;

		$screen_id = false;

		if ( function_exists( 'get_current_screen' ) ) {
			$screen    = get_current_screen();
			$screen_id = isset( $screen, $screen->id ) ? $screen->id : '';
		}

		if ( ! empty( $_REQUEST['screen'] ) ) { // WPCS: input var ok.
			$screen_id = twer_clean( wp_unslash( $_REQUEST['screen'] ) ); // WPCS: input var ok, sanitization ok.
		}

		switch ( $screen_id ) {
			case 'edit-twer-custom-fields':
				include_once 'list-tables/class-twer-admin-list-table-custom-fields.php';
				$twer_list_table = new TWER_Admin_List_Table_Custom_Fields();
				break;
			case 'edit-twer-templates':
				include_once 'list-tables/class-twer-admin-list-table-templates.php';
				$twer_list_table = new TWER_Admin_List_Table_Templates();
				break;
			case 'edit-twer-shapes':
				include_once 'list-tables/class-twer-admin-list-table-shapes.php';
				$twer_list_table = new TWER_Admin_List_Table_Shapes();
				break;
			case 'edit-treweler-import':
				include_once 'list-tables/class-twer-admin-list-table-import.php';
				$twer_list_table = new TWER_Admin_List_Table_Import();
				break;
		}

		// Ensure the table handler is only loaded once. Prevents multiple loads if a plugin calls check_ajax_referer many times.
		remove_action( 'current_screen', [ $this, 'setup_screen' ] );
		remove_action( 'check_ajax_referer', [ $this, 'setup_screen' ] );
	}

	/**
	 * Change messages when a post type is updated.
	 *
	 * @param array $messages Array of messages.
	 *
	 * @return array
	 */
	public function post_updated_messages( $messages ) {
		global $post;

		$messages['twer-custom-fields'] = [
			0  => '', // Unused. Messages start at index 1.
			1  => esc_html__( 'Custom field updated.', 'treweler' ),
			2  => esc_html__( 'Custom field updated.', 'treweler' ),
			3  => esc_html__( 'Custom field deleted.', 'treweler' ),
			4  => esc_html__( 'Custom field updated.', 'treweler' ),
			5  => esc_html__( 'Custom field restored.', 'treweler' ),
			6  => esc_html__( 'Custom field updated.', 'treweler' ),
			7  => esc_html__( 'Custom field saved.', 'treweler' ),
			8  => esc_html__( 'Custom field submitted.', 'treweler' ),
			9  => sprintf(
			/* translators: %s: date */
				'%s %s.',
				esc_html__( 'Custom field scheduled for:', 'treweler' ),
				'<strong>' . date_i18n( esc_html__( 'M j, Y @ G:i', 'treweler' ), strtotime( $post->post_date ) ) . '</strong>'
			),
			10 => esc_html__( 'Custom field draft updated.', 'treweler' )
		];

		$messages['twer-templates'] = [
			0  => '', // Unused. Messages start at index 1.
			1  => esc_html__( 'Template updated.', 'treweler' ),
			2  => esc_html__( 'Template updated.', 'treweler' ),
			3  => esc_html__( 'Template deleted.', 'treweler' ),
			4  => esc_html__( 'Template updated.', 'treweler' ),
			5  => esc_html__( 'Template restored.', 'treweler' ),
			6  => esc_html__( 'Template updated.', 'treweler' ),
			7  => esc_html__( 'Template saved.', 'treweler' ),
			8  => esc_html__( 'Template submitted.', 'treweler' ),
			9  => sprintf(
			/* translators: %s: date */
				'%s %s.',
				esc_html__( 'Template scheduled for:', 'treweler' ),
				'<strong>' . date_i18n( esc_html__( 'M j, Y @ G:i', 'treweler' ), strtotime( $post->post_date ) ) . '</strong>'
			),
			10 => esc_html__( 'Template draft updated.', 'treweler' )
		];

		$messages['twer-shapes'] = [
			0  => '', // Unused. Messages start at index 1.
			1  => esc_html__( 'Shape updated.', 'treweler' ),
			2  => esc_html__( 'Shape updated.', 'treweler' ),
			3  => esc_html__( 'Shape deleted.', 'treweler' ),
			4  => esc_html__( 'Shape updated.', 'treweler' ),
			5  => esc_html__( 'Shape restored.', 'treweler' ),
			6  => esc_html__( 'Shape updated.', 'treweler' ),
			7  => esc_html__( 'Shape saved.', 'treweler' ),
			8  => esc_html__( 'Shape submitted.', 'treweler' ),
			9  => sprintf(
			/* translators: %s: date */
				'%s %s.',
				esc_html__( 'Shape scheduled for:', 'treweler' ),
				'<strong>' . date_i18n( esc_html__( 'M j, Y @ G:i', 'treweler' ), strtotime( $post->post_date ) ) . '</strong>'
			),
			10 => esc_html__( 'Shape draft updated.', 'treweler' )
		];

		$messages['treweler-import'] = [
			0  => '', // Unused. Messages start at index 1.
			1  => esc_html__( 'Import successfully updated.', 'treweler' ),
			2  => esc_html__( 'Custom field updated.', 'treweler' ), // You can change this to be more specific, if needed
			3  => esc_html__( 'Import permanently deleted.', 'treweler' ),
			4  => esc_html__( 'Import successfully updated.', 'treweler' ),
			5  => esc_html__( 'Import restored from the trash.', 'treweler' ),
			6  => esc_html__( 'Import published.', 'treweler' ),
			7  => esc_html__( 'Import saved as a draft.', 'treweler' ),
			8  => esc_html__( 'Import submitted for review.', 'treweler' ),
			9  => sprintf(
			/* translators: %s: Scheduled date */
				esc_html__( 'Import scheduled for: %s.', 'treweler' ),
				'<strong>' . date_i18n( esc_html__( 'M j, Y @ G:i', 'treweler' ), strtotime( $post->post_date ) ) . '</strong>'
			),
			10 => esc_html__( 'Draft import updated.', 'treweler' )
		];

		return $messages;
	}

	/**
	 * Specify custom bulk actions messages for different post types.
	 *
	 * @param array $bulk_messages Array of messages.
	 * @param array $bulk_counts Array of how many objects were updated.
	 *
	 * @return array
	 */
	public function bulk_post_updated_messages( $bulk_messages, $bulk_counts ) {

		$bulk_messages['twer-custom-fields'] = [
			/* translators: %s: custom field count */
			'updated'   => _n( '%s custom field updated.', '%s custom fields updated.', $bulk_counts['updated'], 'treweler' ),
			/* translators: %s: custom field count */
			'locked'    => _n( '%s custom field not updated, somebody is editing it.',
				'%s custom fields not updated, somebody is editing them.', $bulk_counts['locked'], 'treweler' ),
			/* translators: %s: custom field count */
			'deleted'   => _n( '%s custom field permanently deleted.', '%s custom fields permanently deleted.',
				$bulk_counts['deleted'], 'treweler' ),
			/* translators: %s: custom field count */
			'trashed'   => _n( '%s custom field moved to the Trash.', '%s custom fields moved to the Trash.',
				$bulk_counts['trashed'], 'treweler' ),
			/* translators: %s: custom field count */
			'untrashed' => _n( '%s custom field restored from the Trash.', '%s custom fields restored from the Trash.',
				$bulk_counts['untrashed'], 'treweler' ),
		];

		$bulk_messages['twer-templates'] = [
			/* translators: %s: custom field count */
			'updated'   => _n( '%s template updated.', '%s templates updated.', $bulk_counts['updated'], 'treweler' ),
			/* translators: %s: custom field count */
			'locked'    => _n( '%s template not updated, somebody is editing it.',
				'%s templates not updated, somebody is editing them.', $bulk_counts['locked'], 'treweler' ),
			/* translators: %s: custom field count */
			'deleted'   => _n( '%s template permanently deleted.', '%s templates permanently deleted.',
				$bulk_counts['deleted'], 'treweler' ),
			/* translators: %s: custom field count */
			'trashed'   => _n( '%s template moved to the Trash.', '%s templates moved to the Trash.',
				$bulk_counts['trashed'], 'treweler' ),
			/* translators: %s: custom field count */
			'untrashed' => _n( '%s template restored from the Trash.', '%s templates restored from the Trash.',
				$bulk_counts['untrashed'], 'treweler' ),
		];

		$bulk_messages['twer-shapes'] = [
			/* translators: %s: custom field count */
			'updated'   => _n( '%s shape updated.', '%s shapes updated.', $bulk_counts['updated'], 'treweler' ),
			/* translators: %s: custom field count */
			'locked'    => _n( '%s shape not updated, somebody is editing it.',
				'%s shapes not updated, somebody is editing them.', $bulk_counts['locked'], 'treweler' ),
			/* translators: %s: custom field count */
			'deleted'   => _n( '%s shape permanently deleted.', '%s shapes permanently deleted.',
				$bulk_counts['deleted'], 'treweler' ),
			/* translators: %s: custom field count */
			'trashed'   => _n( '%s shape moved to the Trash.', '%s shapes moved to the Trash.',
				$bulk_counts['trashed'], 'treweler' ),
			/* translators: %s: custom field count */
			'untrashed' => _n( '%s shape restored from the Trash.', '%s shapes restored from the Trash.',
				$bulk_counts['untrashed'], 'treweler' ),
		];

		$bulk_messages['treweler-import'] = [
			/* translators: %s: Number of updated imports */
			'updated'   => _n( '%s import updated.', '%s imports updated.', $bulk_counts['updated'], 'treweler' ),
			/* translators: %s: Number of locked imports */
			'locked'    => _n( '%s import is being edited by someone else.', '%s imports are being edited by someone else.', $bulk_counts['locked'], 'treweler' ),
			/* translators: %s: Number of deleted imports */
			'deleted'   => _n( '%s import permanently deleted.', '%s imports permanently deleted.', $bulk_counts['deleted'], 'treweler' ),
			/* translators: %s: Number of trashed imports */
			'trashed'   => _n( '%s import moved to the trash.', '%s imports moved to the trash.', $bulk_counts['trashed'], 'treweler' ),
			/* translators: %s: Number of restored imports */
			'untrashed' => _n( '%s import restored from the trash.', '%s imports restored from the trash.', $bulk_counts['untrashed'], 'treweler' ),
		];

		return $bulk_messages;
	}


	/**
	 * Change title boxes in admin.
	 *
	 * @param string $text Text to shown.
	 * @param WP_Post $post Current post object.
	 *
	 * @return string
	 */
	public function enter_title_here( $text, $post ) {
		switch ( $post->post_type ) {
			case 'twer-custom-fields':
				$text = esc_html__( 'Field Name', 'treweler' );
				break;
			case 'twer-templates':
				$text = esc_html__( 'Template Name', 'treweler' );
				break;
			case 'twer-shapes':
				$text = esc_html__( 'Shape Name', 'treweler' );
				break;
			case 'treweler-import':
				$text = esc_html__( 'Name', 'treweler' );
				break;
		}

		return $text;
	}

	/**
	 * Print map/marker/route init fields.
	 *
	 * @param WP_Post $post Current post object.
	 */
	public function edit_form_after_title( $post ) {
		$post_type = isset( $post->post_type ) ? $post->post_type : '';
		if ( twer_is_valid_apikey() ) {
			echo '<div id="js-twer-body" class="twer-body">';
			if ( 'twer-shapes' === $post_type ) {
				$message_text = '';
				ob_start();
				?>
                <p><?php printf( esc_html__( 'In draw mode :%s
    - Alt + Drag with left click to create nodes with bezier handles.%s
    - Left click to create nodes without handles.', 'treweler' ), '<br>', '<br>' ); ?></p>

                <p><?php printf( esc_html__( 'In edit mode :%s
              - Drag with Left click on a handle to move it.%s
              - Alt + Drag with left click on a handle to break handle symetry.', 'treweler' ), '<br>', '<br>' ); ?></p>
				<?php
				$message_text = ob_get_clean();

				echo '<div id="js-twer-map-msg" class="twer-map-message">' . $message_text . '</div>';
				echo '<div id="js-twer-map" class="twer-map"></div>';
			}

			echo '</div>';
		}
	}
}

new TWER_Admin_Post_Types();
