<?php
/**
 * List tables: Import.
 *
 * @package  Treweler/Admin
 * @version  1.14
 */

namespace TrewelerPro;

use TWER_Admin_List_Table;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( class_exists( 'TWER_Admin_List_Table_Import', false ) ) {
	return;
}

if ( ! class_exists( 'TWER_Admin_List_Table', false ) ) {
	include_once TWER_ABSPATH . 'includes/admin/list-tables/abstract-class-twer-admin-list-table.php';
}

/**
 * TWER_Admin_List_Table_Products Class.
 */
class TWER_Admin_List_Table_Import extends TWER_Admin_List_Table {

	/**
	 * Post type.
	 *
	 * @var string
	 */
	protected $list_table_type = 'treweler-import';

	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct();

		add_filter( 'disable_months_dropdown', '__return_true' );
		add_filter( 'views_edit-treweler-import', array( $this, 'import_views' ) );
	}

	/**
	 * Change views on the edit import screen.
	 *
	 * @param array $views Array of views.
	 *
	 * @return array
	 */
	public function import_views( $views ) {

		unset( $views['mine'] );
		unset( $views['draft'] );
		unset( $views['publish'] );

		return $views;
	}

	/**
	 * Define which columns are sortable.
	 *
	 * @param array $columns Existing columns.
	 *
	 * @return array
	 */
	public function define_sortable_columns( $columns ) {
		$custom = [
			'name' => 'title',
		];

		return wp_parse_args( $custom, $columns );
	}

	/**
	 * Get bulk actions.
	 *
	 * @return array
	 */
	public function define_bulk_actions( $actions ) {
		unset( $actions['edit'] );

		return $actions;
	}


	/**
	 * Define which columns to show on this screen.
	 *
	 * @param array $columns Existing columns.
	 *
	 * @return array
	 */
	public function define_columns( $columns ) {
		if ( empty( $columns ) && ! is_array( $columns ) ) {
			$columns = [];
		}

		unset( $columns['title'], $columns['comments'], $columns['date'], $columns['categories'] );

		$show_columns                = [];
		$show_columns['cb']          = '<input type="checkbox" />';
		$show_columns['name']        = esc_html__( 'Name', 'treweler' );
		$show_columns['last_status'] = esc_html__( 'Last Status', 'treweler' );
		$show_columns['last_import'] = esc_html__( 'Last Import', 'treweler' );
		$show_columns['date']        = esc_html__( 'Date', 'treweler' );

		return array_merge( $show_columns, $columns );
	}


	/**
	 * Define primary column.
	 *
	 * @return string
	 */
	protected function get_primary_column() {
		return 'name';
	}


	/**
	 * Get row actions to show in the list table.
	 *
	 * @param array $actions Array of actions.
	 * @param WP_Post $post Current post object.
	 *
	 * @return array
	 */
	protected function get_row_actions( $actions, $post ) {
		/* translators: %d: post ID. */
		return array_merge( [ 'id' => sprintf( esc_html__( 'ID: %d', 'treweler' ), $post->ID ) ], $actions );
	}


	/**
	 * Render column: name.
	 */
	protected function render_name_column() {
		global $post;

		$edit_link = get_edit_post_link( $this->object->post_id );
		$title     = _draft_or_post_title();

		echo '<strong><a class="row-title" href="' . esc_url( $edit_link ) . '">' . esc_html( $title ) . '</a>';

		_post_states( $post );

		echo '</strong>';

		$parent_id = wp_get_post_parent_id( $this->object->post_id );
		if ( $parent_id > 0 ) {
			echo '&nbsp;&nbsp;&larr; <a href="' . esc_url( get_edit_post_link( $parent_id ) ) . '">' . get_the_title( $parent_id ) . '</a>'; // @codingStandardsIgnoreLine.
		}

		get_inline_data( $post );
	}


	/**
	 * Render columm: last_status.
	 */
	protected function render_last_status_column() {
		$importID   = $this->object->ID ?? 0;
		$lastStatus = get_post_meta( $importID, '_treweler_import_last_status', true );
		if ( ! empty( $lastStatus ) ) {
			$lastStatusReadable = implode( ', ', array_map( static function( $key, $value ) {
				switch ( $key ) {
					case 'imported' :
						$label = esc_html__( 'Imported', 'treweler' );
						break;
					case 'updated' :
						$label = esc_html__( 'Updated', 'treweler' );
						break;
					case 'removed' :
						$label = esc_html__( 'Removed', 'treweler' );
						break;
					default:
						$label = ucfirst( $key );
						break;
				}

				return "$value " . $label;
			}, array_keys( $lastStatus ), $lastStatus ) );

			echo esc_html( $lastStatusReadable );
		}
	}

	/**
	 * Render columm: last_import.
	 */
	protected function render_last_import_column() {
		$importID       = $this->object->ID ?? 0;
		$lastImportDate = get_post_meta( $importID, '_treweler_import_last_import_date', true );

		if ( ! empty( $lastImportDate ) ) {
			printf( '%s<br>%s', esc_html__( 'Imported', 'treweler' ), gmdate( 'Y/m/d \a\t g:i a', $lastImportDate ) );
		}
	}

	/**
	 * Pre-fetch any data for the row each column has access to it. the_marker global is there for bw compat.
	 *
	 * @param int $post_id Post ID being shown.
	 */
	protected function prepare_row_data( $post_id ) {
		global $the_import;

		if ( empty( $this->object ) || $this->object->post_id !== $post_id ) {
			$the_import   = get_post( $post_id );
			$this->object = $the_import;
		}
	}

}
