<?php
/**
 * PRO Enable Upload Difference Mime Types
 * This class provides granular control over file uploads in WordPress, enabling users to securely and
 * efficiently upload, manage, and serve files of various MIME types. It is highly extensible and
 * integrates well with WordPress's native upload infrastructure.
 *
 * @package Treweler/Classes/MimeTypes
 * @version 1.14
 */

namespace TrewelerPro;

defined( 'ABSPATH' ) || exit;

/**
 * Mime types Class.
 */
class TWER_Mime_Types {

	private static array $settings = [];

	public static function init() {
		global $wp_version;

		self::$settings = [
			'mime_type_values'          => 'a:3:{i:0;s:19:"svg = image/svg+xml";i:1;s:23:"json = application/json";i:2;s:25:"gpx = application/gpx+xml";}',
			'security_attempt_enable'   => 'no',
			'filename_sanitized_enable' => 'no',
			'file_type_debug'           => 'no'
		];

		if ( version_compare( $wp_version, '5.1' ) >= 0 ):
			add_filter( 'wp_check_filetype_and_ext', [ __CLASS__, 'add_allow_upload_extension_exception' ], 10, 5 );
		else:
			add_filter( 'wp_check_filetype_and_ext', [ __CLASS__, 'add_allow_upload_extension_exception' ], 10, 4 );
		endif;

		add_filter( 'upload_mimes', [ __CLASS__, 'add_allow_upload_extension' ] );
	}

	public static function add_allow_upload_extension( $mimes ) {
		$mime_type_values = array();

		if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
			require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		}


		if ( ! isset( self::$settings['mime_type_values'] ) ) {
			return $mimes;
		}
		$mime_type_values = maybe_unserialize( self::$settings['mime_type_values'] );

		if ( ! empty( $mime_type_values ) ) {
			foreach ( (array) $mime_type_values as $line ) {
				// Ignore to the right of '#' on a line.
				$line = substr( $line, 0, strcspn( $line, '#' ) );
				// Escape Strings
				$line = wp_strip_all_tags( $line );
				// If 2 or more "=" character in the line data, it will be ignored.
				$line_value = explode( "=", $line );
				if ( count( $line_value ) != 2 ) {
					continue;
				}

				// If the head in each line is set to '-', then the MIME type restricts.
				// ex. -bmp = image/bmp
				// The files which has "bmp" file extention becomes not to be able to upload.
				$line_pref_str = explode( "-", $line_value[0] );
				if ( count( $line_pref_str ) === 2 ):
					if ( isset( $mimes[ trim( $line_pref_str[1] ) ] ) ):
						unset( $mimes[ trim( $line_pref_str[1] ) ] );
					endif;
				else:
					// "　" is the Japanese multi-byte space. If the character is found out, it automatically change the space.
					$mimes[ trim( $line_value[0] ) ] = trim( str_replace( "　", " ", $line_value[1] ) );
				endif;
			}
		}

		return $mimes;
	}

	public static function remove_underscore( $filename, $filename_raw ) {
		return str_replace( "_.", ".", $filename );
	}

	public static function add_allow_upload_extension_exception( $data, $file, $filename, $mimes, $real_mime = null ) {
		$mime_type_values = false;


		if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
			require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		}

		if ( ! isset( self::$settings['mime_type_values'] ) || empty( self::$settings['mime_type_values'] ) ) {
			return $data;
		} else {
			$mime_type_values = maybe_unserialize( self::$settings['mime_type_values'] );
		}

		$ext = $type = $proper_filename = false;
		if ( isset( $data['ext'] ) ) {
			$ext = $data['ext'];
		}
		if ( isset( $data['type'] ) ) {
			$ext = $data['type'];
		}
		if ( isset( $data['proper_filename'] ) ) {
			$ext = $data['proper_filename'];
		}
		if ( $ext != false && $type != false ) {
			return $data;
		}

		// If file extension is 2 or more
		$f_sp        = explode( ".", $filename );
		$f_exp_count = count( $f_sp );

		// Filename type is "XXX" (There is not file extension).
		if ( $f_exp_count <= 1 ) {
			return $data;
			/* Even if the file extension is "XXX.ZZZ", "XXX.YYY.ZZZ", "AAA.XXX.YYY.ZZZ" or more, it always picks up  the tail of the extensions.
			*/
		} else {
			$f_name = $f_sp[0];
			$f_ext  = $f_sp[ $f_exp_count - 1 ];
			// WordPress sanitizes the filename in case of 2 or more extensions.
			// ex. XXX.YYY.ZZZ --> XXX_.YYY.ZZZ.
			// The following function fixes the sanitized extension when a file is uploaded in the media in case of allowed extensions.
			// ex. XXX.YYY.ZZZ -- sanitized --> XXX_.YYY.ZZZ -- fixed the plugin --> XXX.YYY.ZZZ
			// In detail, please see sanitize_file_name function in "wp-includes/formatting.php".
			//var_dump(self::$settings['filename_sanitized_enable']);
			if ( isset( self::$settings['filename_sanitized_enable'] ) && self::$settings['filename_sanitized_enable'] === "yes" ) {
			} else {
				add_filter( 'sanitize_file_name', [__CLASS__, 'remove_underscore'], 10, 2 );
			}
		}

		// If "security_attempt_enable" option disables (default) in the admin menu, the plugin avoids the security check regarding a file extension by WordPress core because of flexible management.
		if ( isset( self::$settings['security_attempt_enable'] ) && self::$settings['security_attempt_enable'] === "yes" ) {
			if ( isset( self::$settings['file_type_debug'] ) && self::$settings['file_type_debug'] === "yes" ):
				$finfo     = finfo_open( FILEINFO_MIME_TYPE );
				$real_mime = finfo_file( $finfo, $file );
				var_dump( __( "WordPress recognizes that the file type is [" . finfo_file( $finfo, $file ) . "].", 'wp-add-mime-types' ) );
				finfo_close( $finfo );
			endif;

			return $data;
		}


		$flag = false;
		if ( ! empty( $mime_type_values ) ) {
			foreach ( (array) $mime_type_values as $line ) {
				// Ignore to the right of '#' on a line.
				$line = substr( $line, 0, strcspn( $line, '#' ) );
				// Escape Strings
				$line = wp_strip_all_tags( $line );

				$line_value = explode( "=", $line );
				if ( count( $line_value ) != 2 ) {
					continue;
				}
				// "　" is the Japanese multi-byte space. If the character is found out, it automatically change the space.
				if ( trim( $line_value[0] ) === $f_ext ) {
					$ext  = $f_ext;
					$type = trim( str_replace( "　", " ", $line_value[1] ) );
					$flag = true;
					break;
				}
			}
		}

		if ( $flag ) {
			return compact( 'ext', 'type', 'proper_filename' );
		} else {
			return $data;
		}
	}
}

TWER_Mime_Types::init();


